@@ -1,6 +1,9 @@
 """
 Tests for pulp
 """
+import os
+import tempfile
+
 from pulp.constants import PulpError
 from pulp.apis import *
 from pulp import LpVariable, LpProblem, lpSum, LpConstraintVar, LpFractionConstraint
@@ -9,6 +12,30 @@ from pulp.tests.bin_packing_problem import create_bin_packing_problem
 from pulp.utilities import makeDict
 import unittest
 
+# from: http://lpsolve.sourceforge.net/5.5/mps-format.htm
+EXAMPLE_MPS_RHS56 = """NAME          TESTPROB
+ROWS
+ N  COST
+ L  LIM1
+ G  LIM2
+ E  MYEQN
+COLUMNS
+    XONE      COST                 1   LIM1                 1
+    XONE      LIM2                 1
+    YTWO      COST                 4   LIM1                 1
+    YTWO      MYEQN               -1
+    ZTHREE    COST                 9   LIM2                 1
+    ZTHREE    MYEQN                1
+RHS
+    RHS1      LIM1                 5   LIM2                10
+    RHS1      MYEQN                7
+BOUNDS
+ UP BND1      XONE                 4
+ LO BND1      YTWO                -1
+ UP BND1      YTWO                 1
+ENDATA
+"""
+
 
 def dumpTestProblem(prob):
     try:
@@ -177,6 +204,9 @@ class BaseSolverTest:
                 GUROBI_CMD,
                 MIPCL_CMD,
                 SCIP_CMD,
+                HiGHS_CMD,
+                XPRESS,
+                XPRESS_CMD,
             ]:
                 try:
                     pulpTestCheck(
@@ -221,6 +251,10 @@ class BaseSolverTest:
                 MIPCL_CMD,
                 MOSEK,
                 SCIP_CMD,
+                HiGHS_CMD,
+                XPRESS,
+                XPRESS_CMD,
+                XPRESS_PY,
             ]:
                 try:
                     pulpTestCheck(
@@ -1039,6 +1073,26 @@ class BaseSolverTest:
             print("\t Testing reading MPS files - maximize")
             self.assertDictEqual(_dict1, _dict2)
 
+        def test_importMPS_noname(self):
+            name = self._testMethodName
+            prob = LpProblem("", const.LpMaximize)
+            x = LpVariable("x", 0, 4)
+            y = LpVariable("y", -1, 1)
+            z = LpVariable("z", 0)
+            w = LpVariable("w", 0)
+            prob += x + 4 * y + 9 * z, "obj"
+            prob += x + y <= 5, "c1"
+            prob += x + z >= 10, "c2"
+            prob += -y + z == 7, "c3"
+            prob += w >= 0, "c4"
+            filename = name + ".mps"
+            prob.writeMPS(filename)
+            _vars, prob2 = LpProblem.fromMPS(filename, sense=prob.sense)
+            _dict1 = getSortedDict(prob)
+            _dict2 = getSortedDict(prob2)
+            print("\t Testing reading MPS files - noname")
+            self.assertDictEqual(_dict1, _dict2)
+
         def test_importMPS_integer(self):
             name = self._testMethodName
             prob = LpProblem(name, const.LpMinimize)
@@ -1076,6 +1130,14 @@ class BaseSolverTest:
             print("\t Testing reading MPS files - binary variable, no constraint names")
             self.assertDictEqual(_dict1, _dict2)
 
+        def test_importMPS_RHS_fields56(self):
+            """Import MPS file with RHS definitions in fields 5 & 6."""
+            with tempfile.NamedTemporaryFile(delete=False) as h:
+                h.write(str.encode(EXAMPLE_MPS_RHS56))
+            _, problem = LpProblem.fromMPS(h.name)
+            os.unlink(h.name)
+            self.assertEqual(problem.constraints["LIM2"].constant, -10)
+
         # def test_importMPS_2(self):
         #     name = self._testMethodName
         #     # filename = name + ".mps"
@@ -1280,6 +1342,71 @@ class BaseSolverTest:
             #        indexs=(customers, agents),
             #    )
 
+        def test_parse_cplex_mipopt_solution(self):
+            """
+            Ensures `readsol` can parse CPLEX mipopt solutions (see issue #508).
+            """
+            try:
+                from io import StringIO
+            except ImportError:  # python 2
+                from StringIO import StringIO
+
+            print("\t Testing that `readsol` can parse CPLEX mipopt solution")
+            # Example solution generated by CPLEX mipopt solver
+            file_content = """<?xml version = "1.0" encoding="UTF-8" standalone="yes"?>
+                <CPLEXSolution version="1.2">
+                <header
+                    problemName="mipopt_solution_example.lp"
+                    solutionName="incumbent"
+                    solutionIndex="-1"
+                    objectiveValue="442"
+                    solutionTypeValue="3"
+                    solutionTypeString="primal"
+                    solutionStatusValue="101"
+                    solutionStatusString="integer optimal solution"
+                    solutionMethodString="mip"
+                    primalFeasible="1"
+                    dualFeasible="1"
+                    MIPNodes="25471"
+                    MIPIterations="282516"
+                    writeLevel="1"/>
+                <quality
+                    epInt="1.0000000000000001e-05"
+                    epRHS="9.9999999999999995e-07"
+                    maxIntInfeas="8.8817841970012523e-16"
+                    maxPrimalInfeas="0"
+                    maxX="48"
+                maxSlack="141"/>
+                <linearConstraints>
+                    <constraint name="C1" index="0" slack="0"/>
+                    <constraint name="C2" index="1" slack="0"/>
+                </linearConstraints>
+                <variables>
+                    <variable name="x" index="0" value="42"/>
+                    <variable name="y" index="1" value="0"/>
+                </variables>
+                <objectiveValues>
+                    <objective index="0" name="x" value="42"/>
+                </objectiveValues>
+                </CPLEXSolution>
+            """
+
+            try:
+                solution_file = StringIO(file_content)
+            except TypeError:  # python 2
+                solution_file = StringIO(unicode(file_content))
+
+            # This call to `readsol` would crash for this solution format #508
+            _, _, reducedCosts, shadowPrices, _, _ = CPLEX_CMD().readsol(solution_file)
+
+            # Because mipopt solutions have no `reducedCost` fields
+            # it should be all None
+            self.assertTrue(all(c is None for c in reducedCosts.values()))
+
+            # Because mipopt solutions have no `shadowPrices` fields
+            # it should be all None
+            self.assertTrue(all(c is None for c in shadowPrices.values()))
+
 
 class PULP_CBC_CMDTest(BaseSolverTest.PuLPTest):
     solveInst = PULP_CBC_CMD
@@ -1293,8 +1420,12 @@ class CPLEX_PYTest(BaseSolverTest.PuLPTest):
     solveInst = CPLEX_CMD
 
 
-class XPRESSTest(BaseSolverTest.PuLPTest):
-    solveInst = XPRESS
+class XPRESS_CMDTest(BaseSolverTest.PuLPTest):
+    solveInst = XPRESS_CMD
+
+
+class XPRESS_PyTest(BaseSolverTest.PuLPTest):
+    solveInst = XPRESS_PY
 
 
 class COIN_CMDTest(BaseSolverTest.PuLPTest):
@@ -1341,6 +1472,10 @@ class SCIP_CMDTest(BaseSolverTest.PuLPTest):
     solveInst = SCIP_CMD
 
 
+class HiGHS_CMDTest(BaseSolverTest.PuLPTest):
+    solveInst = HiGHS_CMD
+
+
 def pulpTestCheck(
     prob,
     solver,
@@ -1349,7 +1484,7 @@ def pulpTestCheck(
     reducedcosts=None,
     duals=None,
     slacks=None,
-    eps=10 ** -3,
+    eps=10**-3,
     status=None,
     objective=None,
     **kwargs
